@@ -388,6 +388,22 @@ static void init_xlocale (void);
 #else
 #define init_xlocale() ((void)0)
 #endif
+
+/* override gcc attribute __wur used in glibc */
+static inline int
+nowarn_chown (const char *file, uid_t uid, gid_t gid)
+{
+  return chown (file, uid, gid);
+}
+#define chown(file,uid,gid) (nowarn_chown((file),(uid),(gid)))
+
+static inline int
+nowarn_fchown (int fd, uid_t uid, gid_t gid)
+{
+  return fchown (fd, uid, gid);
+}
+#define fchown(fd,uid,gid) (nowarn_fchown((fd),(uid),(gid)))
+
 /*----------------------------------------------------------------------*/
 /*}}} */
 
@@ -504,7 +520,8 @@ clean_exit (void)
 #endif
   privileges (RESTORE);
   chmod (ttydev, ttyfd_stat.st_mode);
-  chown (ttydev, ttyfd_stat.st_uid, ttyfd_stat.st_gid);
+  /* fail silently, e.g., when installed without sticky bit on */
+  (void) chown (ttydev, ttyfd_stat.st_uid, ttyfd_stat.st_gid);
   cleanutent ();
   privileges (IGNORE);
 }
@@ -647,27 +664,31 @@ get_tty (void)
 #endif /* USE_GETGRNAME */
 
     privileges (RESTORE);
-    fchown (fd, getuid (), gid);	/* fail silently */
+    (void) fchown (fd, getuid (), gid);	/* fail silently */
     fchmod (fd, mode);
     privileges (IGNORE);
   }
 #endif /* __svr4__ */
 
-  /*
-   * Close all file descriptors.  If only stdin/out/err are closed,
-   * child processes remain alive upon deletion of the window.
-   */
   {
     int i;
+    /*
+     * Close all file descriptors.  If only stdin/out/err are closed,
+     * child processes remain alive upon deletion of the window.
+     */
     for (i = 0; i < num_fds; i++)
       if (i != fd)
 	close (i);
-  }
 
-  /* Reopen stdin, stdout and stderr over the tty file descriptor */
-  dup (fd);			/* 0: stdin */
-  dup (fd);			/* 1: stdout */
-  dup (fd);			/* 2: stderr */
+    /* Reopen stdin, stdout and stderr over the tty file descriptor */
+    for (i = 0; i < 3; i++)
+      if (dup (fd) < 0)
+        {
+          fprintf (stderr,
+                   "Fail to dup tty for fd %d: %s\n", i, strerror (errno));
+          exit (EXIT_FAILURE);
+        }
+  }
 
   if (fd > 2)
     close (fd);
